﻿/*
  Example Visual Studio C# code for controlling the IOM2 USB relay module by interfacing to the IOM2.dll file. The IOM2.dll file 
  will handle all of the serial communications with the module. 

  Copy IOM2.dll into the project folder and add to the project via Add>Existing Item. In Properties for the dll, set Build Action
  to None and Copy to Output Directory to Copy always.

  The number of the virtual COM port created upon installation of the module must be known. That number must be either passed
  to Start_IOM2 or selected via SelectPort_IOM2 in order to open the port and initiate communications.

  Before terminating the program, Stop_IOM2 should be called to stop communications, close the COM port, and gracefully clean up
  any resources created.

  In between Start_IOM2 or SelectPort_IOM2 and Stop_IOM2, a variety of functions can be called including those to control the
  relays or check the state of the inputs.

  Copyright © 2024 BadgerWare, LLC. All rights reserved.
  www.badgerware.net
  www.usb-relay.com
*/

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Runtime.InteropServices;   // Required for DllImport to be used

namespace IOM2_DLL_Interface_CSharp
{    public partial class Form1 : Form
    {
        // Imports and prototypes for routines available in IOM2.dll
        [DllImport("IOM2.dll", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
        public static extern void Start_IOM2(int port);
        [DllImport("IOM2.dll", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
        public static extern int SelectPort_IOM2();
        [DllImport("IOM2.dll", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
        public static extern int GetStatus_IOM2();
        [DllImport("IOM2.dll", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
        public static extern bool GetInput1_IOM2();
        [DllImport("IOM2.dll", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
        public static extern bool GetInput2_IOM2();
        [DllImport("IOM2.dll", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
        public static extern bool GetInput3_IOM2();
        [DllImport("IOM2.dll", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
        public static extern bool GetInput4_IOM2();
        [DllImport("IOM2.dll", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
        public static extern bool GetInput5_IOM2();
        [DllImport("IOM2.dll", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
        public static extern bool GetInput6_IOM2();
        [DllImport("IOM2.dll", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
        public static extern bool GetInput7_IOM2();
        [DllImport("IOM2.dll", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
        public static extern bool GetInput8_IOM2();
        [DllImport("IOM2.dll", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
        public static extern void SetRelays_IOM2(bool R1, bool R2, bool R3, bool R4, bool R5, bool R6, bool R7, bool R8, int addr = 0);
        [DllImport("IOM2.dll", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
        public static extern void SetRelay1_IOM2(bool state, int addr = 0);
        [DllImport("IOM2.dll", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
        public static extern void SetRelay2_IOM2(bool state, int addr = 0);
        [DllImport("IOM2.dll", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
        public static extern void SetRelay3_IOM2(bool state, int addr = 0);
        [DllImport("IOM2.dll", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
        public static extern void SetRelay4_IOM2(bool state, int addr = 0);
        [DllImport("IOM2.dll", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
        public static extern void SetRelay5_IOM2(bool state, int addr = 0);
        [DllImport("IOM2.dll", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
        public static extern void SetRelay6_IOM2(bool state, int addr = 0);
        [DllImport("IOM2.dll", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
        public static extern void SetRelay7_IOM2(bool state, int addr = 0);
        [DllImport("IOM2.dll", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
        public static extern void SetRelay8_IOM2(bool state, int addr = 0);
        [DllImport("IOM2.dll", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
        public static extern int GetNumRelays_IOM2();
        [DllImport("IOM2.dll", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
        public static extern void Test_IOM2();
        [DllImport("IOM2.dll", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
        public static extern void ShowDllVersion_IOM2();
        [DllImport("IOM2.dll", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
        public static extern void Stop_IOM2();

        public Form1()
        {
            InitializeComponent();
        }

        /*
            If the application knows the COM port number (e.g.can retrieve it from a file or the application provides its own selection of
            the port number), call Start_IOM2 with the known port number as the argument.
        */
        private void StartBtn_Click(object sender, EventArgs e)
        {
            Start_IOM2(ComPortBox.SelectedIndex);   // Opens the COM port and initiates communications
        }

        /*
            As an alternative to Start_IOM2, SelectPort_IOM2 can be called. A window providing COM port selection will be displayed.
        */
        private void PortBtn_Click(object sender, EventArgs e)
        {
            ComPortBox.SelectedIndex = SelectPort_IOM2();   // Displays the selection window, opens the selected COM port, and initiates communications
        }

        private void GetNumRelaysBtn_Click(object sender, EventArgs e)
        {
            GetNumRelaysBtn.Text = GetNumRelays_IOM2().ToString();  // Queries the module for the number of relays available (0 if comms issue)
        }

        private void TestBtn_Click(object sender, EventArgs e)
        {
            Test_IOM2();    // Displays a window providing an interface to allow the end user to manually test the hardware and external connections
        }

        private void ShowDllVersionBtn_Click(object sender, EventArgs e)
        {
            ShowDllVersion_IOM2();  // Displays a window showing the IOM2.dll version information
        }

        /*
            On command call to Stop_IOM2 for demonstration purposes. In most cases, it's probably best to call it when the application is
            about to terminate.
        */
        private void StopBtn_Click(object sender, EventArgs e)
        {
            Stop_IOM2();    // Stops communications, closes the COM port, and gracefully cleans up any resources created
        }

        /*
            250 msec timer to continuously check communications status and query the states of the inputs. If the inputs are not used, the
            states do not need to be queried. It's possible to just blindly turn the relays on and off without checking the communications
            status, but probably best to know if there is an issue.
        */
        private void UpdateTimer_Tick(object sender, EventArgs e)
        {
            switch (GetStatus_IOM2())
            {
                case -1:
                    StatusLabel.Text = "IOM2 Status: Unknown";
                    break;
                case 0:
                    StatusLabel.Text = "IOM2 Status: Ok";
                    break;
                case 1:
                    StatusLabel.Text = "IOM2 Status: COM Port is off";
                    break;
                case 2:
                    StatusLabel.Text = "IOM2 Status: COM Port conflict";
                    break;
                case 3:
                    StatusLabel.Text = "IOM2 Status: No connection";
                    break;
                case 4:
                    StatusLabel.Text = "IOM2 Status: Parsing error";
                    break;
            }

            // Query the states of each of the inputs.
            Input1Label.Visible = GetInput1_IOM2();
            Input2Label.Visible = GetInput2_IOM2();
            Input3Label.Visible = GetInput3_IOM2();
            Input4Label.Visible = GetInput4_IOM2();
            Input5Label.Visible = GetInput5_IOM2();
            Input6Label.Visible = GetInput6_IOM2();
            Input7Label.Visible = GetInput7_IOM2();
            Input8Label.Visible = GetInput8_IOM2();
        }

        private void Form1_FormClosed(object sender, FormClosedEventArgs e)
        {
            Stop_IOM2();    // Stops communications, closes the COM port, and gracefully cleans up any resources created
        }

        /*
            Set the states of each of the relays with a single command.
        */
        private void SetRelaysBtn_Click(object sender, EventArgs e)
        {
            SetRelays_IOM2(Relay1Box.Checked, Relay2Box.Checked, Relay3Box.Checked, Relay4Box.Checked, Relay5Box.Checked, Relay6Box.Checked, Relay7Box.Checked, Relay8Box.Checked);
        }

        private void Relay1OnBtn_Click(object sender, EventArgs e)
        {
            SetRelay1_IOM2(true);   // Turn relay 1 on
        }

        private void Relay1OffBtn_Click(object sender, EventArgs e)
        {
            SetRelay1_IOM2(false);   // Turn relay 1 off
        }

        private void Relay2OnBtn_Click(object sender, EventArgs e)
        {
            SetRelay2_IOM2(true);   // Turn relay 2 on
        }

        private void Relay2OffBtn_Click(object sender, EventArgs e)
        {
            SetRelay2_IOM2(false);   // Turn relay 2 off
        }

        private void Relay3OnBtn_Click(object sender, EventArgs e)
        {
            SetRelay3_IOM2(true);   // Turn relay 3 on
        }

        private void Relay3OffBtn_Click(object sender, EventArgs e)
        {
            SetRelay3_IOM2(false);   // Turn relay 3 off
        }

        private void Relay4OnBtn_Click(object sender, EventArgs e)
        {
            SetRelay4_IOM2(true);   // Turn relay 4 on
        }

        private void Relay4OffBtn_Click(object sender, EventArgs e)
        {
            SetRelay4_IOM2(false);   // Turn relay 4 off
        }

        private void Relay5OnBtn_Click(object sender, EventArgs e)
        {
            SetRelay5_IOM2(true);   // Turn relay 5 on
        }

        private void Relay5OffBtn_Click(object sender, EventArgs e)
        {
            SetRelay5_IOM2(false);   // Turn relay 5 off
        }

        private void Relay6OnBtn_Click(object sender, EventArgs e)
        {
            SetRelay6_IOM2(true);   // Turn relay 6 on
        }

        private void Relay6OffBtn_Click(object sender, EventArgs e)
        {
            SetRelay6_IOM2(false);   // Turn relay 6 off
        }

        private void Relay7OnBtn_Click(object sender, EventArgs e)
        {
            SetRelay7_IOM2(true);   // Turn relay 7 on
        }

        private void Relay7OffBtn_Click(object sender, EventArgs e)
        {
            SetRelay7_IOM2(false);   // Turn relay 7 off
        }

        private void Relay8OnBtn_Click(object sender, EventArgs e)
        {
            SetRelay8_IOM2(true);   // Turn relay 8 on
        }

        private void Relay8OffBtn_Click(object sender, EventArgs e)
        {
            SetRelay8_IOM2(false);   // Turn relay 8 off
        }
    }
}
