unit IOM2_DLL_Example;
{
  Example Delphi code for controlling the IOM2 USB relay module by interfacing to the IOM2.dll file. The IOM2.dll file will
  handle all of the serial communications with the module.

  Copy IOM2.dll to the same folder as the application .exe file.

  The number of the virtual COM port created upon installation of the module must be known. That number must be either passed
  to Start_IOM2 or selected via SelectPort_IOM2 in order to open the port and initiate communications.

  Before terminating the program, Stop_IOM2 should be called to stop communications, close the COM port, and gracefully clean up
  any resources created.

  In between Start_IOM2 or SelectPort_IOM2 and Stop_IOM2, a variety of functions can be called including those to control the
  relays or check the state of the inputs.

  Copyright  2024 BadgerWare, LLC. All rights reserved.
  www.badgerware.net
  www.usb-relay.com
}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Buttons, ExtCtrls;

type
  TIOM2_DLL_ExampleForm = class(TForm)
    StartBtn: TBitBtn;
    PortBtn: TBitBtn;
    StopBtn: TBitBtn;
    StatusPanel: TPanel;
    UpdateTimer: TTimer;
    ActivePanel: TPanel;
    Input1Panel: TPanel;
    Input2Panel: TPanel;
    Input3Panel: TPanel;
    Input4Panel: TPanel;
    Input5Panel: TPanel;
    Input6Panel: TPanel;
    Input7Panel: TPanel;
    Input8Panel: TPanel;
    Relay1Box: TCheckBox;
    Relay2Box: TCheckBox;
    Relay4Box: TCheckBox;
    Relay3Box: TCheckBox;
    Relay5Box: TCheckBox;
    Relay6Box: TCheckBox;
    Relay7Box: TCheckBox;
    Relay8Box: TCheckBox;
    SetRelaysBtn: TBitBtn;
    Relay1OnBtn: TBitBtn;
    Relay1OffBtn: TBitBtn;
    Relay2OnBtn: TBitBtn;
    Relay2OffBtn: TBitBtn;
    Relay3OnBtn: TBitBtn;
    Relay3OffBtn: TBitBtn;
    Relay4OnBtn: TBitBtn;
    Relay4OffBtn: TBitBtn;
    Relay5OnBtn: TBitBtn;
    Relay5OffBtn: TBitBtn;
    Relay6OnBtn: TBitBtn;
    Relay6OffBtn: TBitBtn;
    Relay7OnBtn: TBitBtn;
    Relay7OffBtn: TBitBtn;
    Relay8OnBtn: TBitBtn;
    Relay8OffBtn: TBitBtn;
    ShowDllVersionBtn: TBitBtn;
    GetNumRelaysBtn: TBitBtn;
    TestBtn: TBitBtn;
    ComPortLabel: TLabel;
    ComPortBox: TComboBox;
    procedure StartBtnClick(Sender: TObject);
    procedure PortBtnClick(Sender: TObject);
    procedure StopBtnClick(Sender: TObject);
    procedure UpdateTimerTimer(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure SetRelaysBtnClick(Sender: TObject);
    procedure Relay1OnBtnClick(Sender: TObject);
    procedure Relay1OffBtnClick(Sender: TObject);
    procedure Relay2OnBtnClick(Sender: TObject);
    procedure Relay2OffBtnClick(Sender: TObject);
    procedure Relay3OnBtnClick(Sender: TObject);
    procedure Relay3OffBtnClick(Sender: TObject);
    procedure Relay4OnBtnClick(Sender: TObject);
    procedure Relay4OffBtnClick(Sender: TObject);
    procedure Relay5OnBtnClick(Sender: TObject);
    procedure Relay5OffBtnClick(Sender: TObject);
    procedure Relay6OnBtnClick(Sender: TObject);
    procedure Relay6OffBtnClick(Sender: TObject);
    procedure Relay7OnBtnClick(Sender: TObject);
    procedure Relay7OffBtnClick(Sender: TObject);
    procedure Relay8OnBtnClick(Sender: TObject);
    procedure Relay8OffBtnClick(Sender: TObject);
    procedure ShowDllVersionBtnClick(Sender: TObject);
    procedure GetNumRelaysBtnClick(Sender: TObject);
    procedure TestBtnClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  IOM2_DLL_ExampleForm: TIOM2_DLL_ExampleForm;

// Prototypes for routines available in IOM2.dll
procedure Start_IOM2(port: integer); stdcall; external 'IOM2.dll'
function SelectPort_IOM2: integer; stdcall; external 'IOM2.dll'
function GetStatus_IOM2: integer; stdcall; external 'IOM2.dll'
function GetInput1_IOM2: boolean; stdcall; external 'IOM2.dll';
function GetInput2_IOM2: boolean; stdcall; external 'IOM2.dll';
function GetInput3_IOM2: boolean; stdcall; external 'IOM2.dll';
function GetInput4_IOM2: boolean; stdcall; external 'IOM2.dll';
function GetInput5_IOM2: boolean; stdcall; external 'IOM2.dll';
function GetInput6_IOM2: boolean; stdcall; external 'IOM2.dll';
function GetInput7_IOM2: boolean; stdcall; external 'IOM2.dll';
function GetInput8_IOM2: boolean; stdcall; external 'IOM2.dll';
procedure SetRelays_IOM2(R1: boolean; R2: boolean; R3: boolean; R4: boolean; R5: boolean; R6: boolean; R7: boolean; R8: boolean; addr: integer = 0); stdcall; external 'IOM2.dll';
procedure SetRelay1_IOM2(state: boolean; addr: integer = 0); stdcall; external 'IOM2.dll';
procedure SetRelay2_IOM2(state: boolean; addr: integer = 0); stdcall; external 'IOM2.dll';
procedure SetRelay3_IOM2(state: boolean; addr: integer = 0); stdcall; external 'IOM2.dll';
procedure SetRelay4_IOM2(state: boolean; addr: integer = 0); stdcall; external 'IOM2.dll';
procedure SetRelay5_IOM2(state: boolean; addr: integer = 0); stdcall; external 'IOM2.dll';
procedure SetRelay6_IOM2(state: boolean; addr: integer = 0); stdcall; external 'IOM2.dll';
procedure SetRelay7_IOM2(state: boolean; addr: integer = 0); stdcall; external 'IOM2.dll';
procedure SetRelay8_IOM2(state: boolean; addr: integer = 0); stdcall; external 'IOM2.dll';
function GetNumRelays_IOM2: integer; stdcall; external 'IOM2.dll';
procedure Test_IOM2; stdcall; external 'IOM2.dll'
procedure ShowDllVersion_IOM2; stdcall; external 'IOM2.dll'
procedure Stop_IOM2; stdcall; external 'IOM2.dll'

implementation

{$R *.DFM}

{
  If the application knows the COM port number (e.g. can retrieve it from a file or the application provides its own selection of
  the port number), call Start_IOM2 with the known port number as the argument.
}
procedure TIOM2_DLL_ExampleForm.StartBtnClick(Sender: TObject);
begin
  Start_IOM2(ComPortBox.ItemIndex); // Opens the COM port and initiates communications
end;

{
  As an alternative to Start_IOM2, SelectPort_IOM2 can be called. A window providing COM port selection will be displayed.
}
procedure TIOM2_DLL_ExampleForm.PortBtnClick(Sender: TObject);
begin
  ComPortBox.ItemIndex := SelectPort_IOM2;  // Displays the selection window, opens the selected COM port, and initiates communications
end;

procedure TIOM2_DLL_ExampleForm.GetNumRelaysBtnClick(Sender: TObject);
begin
  GetNumRelaysBtn.Caption := IntToStr(GetNumRelays_IOM2); // Queries the module for the number of relays available (0 if comms issue)
end;

procedure TIOM2_DLL_ExampleForm.TestBtnClick(Sender: TObject);
begin
  Test_IOM2; // Displays a window providing an interface to allow the end user to manually test the hardware and external connections
end;

procedure TIOM2_DLL_ExampleForm.ShowDllVersionBtnClick(Sender: TObject);
begin
  ShowDllVersion_IOM2;  // Displays a window showing the IOM2.dll version information
end;

{
  On command call to Stop_IOM2 for demonstration purposes. In most cases, it's probably best to call it when the application is
  about to terminate.
}
procedure TIOM2_DLL_ExampleForm.StopBtnClick(Sender: TObject);
begin
  Stop_IOM2; // Stops communications, closes the COM port, and gracefully cleans up any resources created
end;

{
  250 msec timer to continuously check communications status and query the states of the inputs. If the inputs are not used, the
  states do not need to be queried. It's possible to just blindly turn the relays on and off without checking the communications
  status, but probably best to know if there is an issue.
}
procedure TIOM2_DLL_ExampleForm.UpdateTimerTimer(Sender: TObject);
begin
  case GetStatus_IOM2 of  // Query status of the communications.
    -1: StatusPanel.Caption := 'IOM2 Status: Unknown';
    0:  StatusPanel.Caption := 'IOM2 Status: Ok';
    1:  StatusPanel.Caption := 'IOM2 Status: COM Port is off';
    2:  StatusPanel.Caption := 'IOM2 Status: COM Port conflict';
    3:  StatusPanel.Caption := 'IOM2 Status: No connection';
    4:  StatusPanel.Caption := 'IOM2 Status: Parsing error';
  end;

  // Query the states of each of the inputs.
  Input1Panel.Visible := GetInput1_IOM2;
  Input2Panel.Visible := GetInput2_IOM2;
  Input3Panel.Visible := GetInput3_IOM2;
  Input4Panel.Visible := GetInput4_IOM2;
  Input5Panel.Visible := GetInput5_IOM2;
  Input6Panel.Visible := GetInput6_IOM2;
  Input7Panel.Visible := GetInput7_IOM2;
  Input8Panel.Visible := GetInput8_IOM2;
end;

procedure TIOM2_DLL_ExampleForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  Stop_IOM2;  // Stops communications, closes the COM port, and gracefully cleans up any resources created
end;

{
  Set the states of each of the relays with a single command.
}
procedure TIOM2_DLL_ExampleForm.SetRelaysBtnClick(Sender: TObject);
begin
  SetRelays_IOM2(Relay1Box.Checked, Relay2Box.Checked, Relay3Box.Checked, Relay4Box.Checked, Relay5Box.Checked, Relay6Box.Checked, Relay7Box.Checked, Relay8Box.Checked);
end;

procedure TIOM2_DLL_ExampleForm.Relay1OnBtnClick(Sender: TObject);
begin
  SetRelay1_IOM2(true); // Turn relay 1 on
end;

procedure TIOM2_DLL_ExampleForm.Relay1OffBtnClick(Sender: TObject);
begin
  SetRelay1_IOM2(false);  // Turn relay 1 off
end;

procedure TIOM2_DLL_ExampleForm.Relay2OnBtnClick(Sender: TObject);
begin
  SetRelay2_IOM2(true); // Turn relay 2 on
end;

procedure TIOM2_DLL_ExampleForm.Relay2OffBtnClick(Sender: TObject);
begin
  SetRelay2_IOM2(false);  // Turn relay 2 off
end;

procedure TIOM2_DLL_ExampleForm.Relay3OnBtnClick(Sender: TObject);
begin
  SetRelay3_IOM2(true); // Turn relay 3 on
end;

procedure TIOM2_DLL_ExampleForm.Relay3OffBtnClick(Sender: TObject);
begin
  SetRelay3_IOM2(false);  // Turn relay 3 off
end;

procedure TIOM2_DLL_ExampleForm.Relay4OnBtnClick(Sender: TObject);
begin
  SetRelay4_IOM2(true); // Turn relay 4 on
end;

procedure TIOM2_DLL_ExampleForm.Relay4OffBtnClick(Sender: TObject);
begin
  SetRelay4_IOM2(false);  // Turn relay 4 off
end;

procedure TIOM2_DLL_ExampleForm.Relay5OnBtnClick(Sender: TObject);
begin
  SetRelay5_IOM2(true); // Turn relay 5 on
end;

procedure TIOM2_DLL_ExampleForm.Relay5OffBtnClick(Sender: TObject);
begin
  SetRelay5_IOM2(false);  // Turn relay 5 off
end;

procedure TIOM2_DLL_ExampleForm.Relay6OnBtnClick(Sender: TObject);
begin
  SetRelay6_IOM2(true); // Turn relay 6 on
end;

procedure TIOM2_DLL_ExampleForm.Relay6OffBtnClick(Sender: TObject);
begin
  SetRelay6_IOM2(false);  // Turn relay 6 off
end;

procedure TIOM2_DLL_ExampleForm.Relay7OnBtnClick(Sender: TObject);
begin
  SetRelay7_IOM2(true); // Turn relay 7 on
end;

procedure TIOM2_DLL_ExampleForm.Relay7OffBtnClick(Sender: TObject);
begin
  SetRelay7_IOM2(false);  // Turn relay 7 off
end;

procedure TIOM2_DLL_ExampleForm.Relay8OnBtnClick(Sender: TObject);
begin
  SetRelay8_IOM2(true); // Turn relay 8 on
end;

procedure TIOM2_DLL_ExampleForm.Relay8OffBtnClick(Sender: TObject);
begin
  SetRelay8_IOM2(false);  // Turn relay 8 off
end;

end.
